library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;
use ieee.math_real.all;

entity p17_fir_seriell_bench is
end p17_fir_seriell_bench;

architecture Behavioral of p17_fir_seriell_bench is

component p17_fir_seriell is port(
	clk : in std_logic;
	data_in : in std_logic_vector(11 downto 0); -- 12 Bit unsigned
	data_in_new : in std_logic;
	data_out : out std_logic_vector(11 downto 0);
	data_out_new : out std_logic);
end component;

signal clk : std_logic:='0';
signal data_in : std_logic_vector(11 downto 0):=(others => '0');
signal data_in_new : std_logic:='0';
signal data_out : std_logic_vector(11 downto 0):=(others => '0');
signal data_out_new : std_logic:='0';

type rom128x12 is array (0 to 63) of signed(11 downto 0); 
signal sinus_rom : rom128x12;

signal counter: unsigned(5 downto 0):=(others => '0');
signal samplecounter: unsigned(5 downto 0):=(others => '0');
signal speed: unsigned(4 downto 0):="00001";

signal clk_divider: integer range 0 to 99:=0;

signal data_out_buff : std_logic_vector(11 downto 0):=(others => '0');

begin

tabel : for I in 0 to 63 generate
	sinus_rom(I) <= to_signed(integer(sin(2.0*MATH_PI*(real(I)+0.5)/64.0)*2047.5),12);
end generate;

uut: p17_fir_seriell port map(
	clk => clk,
	data_in => data_in, -- 12 Bit unsigned
	data_in_new => data_in_new,
	data_out => data_out_buff,
	data_out_new => data_out_new);

clk <= not clk after 5 ns;

process begin
	wait until rising_edge(clk);
	clk_divider <= clk_divider +1;
	data_in_new <= '0';
	if clk_divider = 99 then
		data_in_new <= '1';
		clk_divider <= 0;
		samplecounter <= samplecounter +1;
		counter <= counter + speed;
		if samplecounter = 63 then
			speed <= speed+1;
		end if;
		data_in <= std_logic_vector(unsigned(sinus_rom(to_integer(counter)) + 2**11)); --unsigned
		if speed = 31 and samplecounter = 63 then
			wait;
		end if;
	end if;
	
	if data_out_new = '1' then
		data_out <= data_out_buff;
	end if;
end process;

end Behavioral;