library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

entity p17_fir_seriell is port(
	clk : in std_logic;
	data_in : in std_logic_vector(11 downto 0); -- 12 Bit unsigned
	data_in_new : in std_logic;
	data_out : out std_logic_vector(11 downto 0);
	data_out_new : out std_logic);
end p17_fir_seriell;

architecture behav of p17_fir_seriell is

constant filterordnung : integer := 16;

type koeffizienten_rom_8 is array (0 to 7) of integer range -2**11 to 2**11-1;
-- LP Barthann F_c=0.1, F_pb=0.18, F_sb=0.2
constant lp_8_koeffizienten : koeffizienten_rom_8 :=(57,291,682,1018,1018,682,291,57);

type koeffizienten_rom_16 is array (0 to 15) of integer range -2**11 to 2**11-1;
-- LP Barthann F_c=0.1, F_pb=0.18, F_sb=0.2
constant lp_16_koeffizienten : koeffizienten_rom_16 :=(-10,-26,-22,42,193,415,648,807,807,648,415,193,42,-22,-26,-10);
-- BP Barthann F_c=0.1, F_c2=0.25, F_sb=0.18, F_pb=0.2, F_pb2=0.25, F_sb2=0.27
constant bp_16_koeffizienten : koeffizienten_rom_16 :=(3,3,78,58,-388,-761,-88,1117,1117,-88,-761,-388,58,78,3,3);

type koeffizienten_rom_64 is array (0 to 63) of integer range -2**11 to 2**11-1;
-- LP Barthann F_c=0.1, F_pb=0.18, F_sb=0.2
constant lp_64_koeffizienten : koeffizienten_rom_64 :=(
1,0,-1,-3,-5,-5,-3,3,10,15,15,7,-8,-24,-35,-33,
-14,17,50,71,66,29,-34,-103,-150,-145,-67,84,290,511,698,809,
809,698,511,290,84,-67,-145,-150,-103,-34,29,66,71,50,17,-14,
-33,-35,-24,-8,7,15,15,10,3,-3,-5,-5,-3,-1,0,1);
-- BP Barthann F_c=0.1, F_c2=0.25, F_sb=0.18, F_pb=0.2, F_pb2=0.25, F_sb2=0.27
constant bp_64_koeffizienten : koeffizienten_rom_64 :=(
-1,-1,2,4,1,1,7,4,-17,-24,-2,8,-10,3,57,59,
-18,-53,-6,-20,-122,-95,110,192,44,18,221,109,-544,-873,-88,1045,
1045,-88,-873,-544,109,221,18,44,192,110,-95,-122,-20,-6,-53,-18,
59,57,3,-10,8,-2,-24,-17,4,7,1,1,4,2,-1,-1);

signal sum: integer range -2**31 to 2**31-1:=0;
type ram_16x12 is array (0 to filterordnung-1) of std_logic_vector(11 downto 0);	
signal data_ram : ram_16x12 :=(others => x"000");
signal counter : integer range 0 to 99:=0; 

begin

process begin
	wait until rising_edge(CLK);
	data_out_new <= '0';
	
	if counter < filterordnung +1 then
		counter <= counter +1;
	end if;
	
	if data_in_new = '1' then
		counter <= 0;
		data_ram(0) <= data_in;
		data_ram(1 to filterordnung-1) <= data_ram(0 to filterordnung-2);
		sum <= 0;
	end if;

	if counter < filterordnung then
		sum <= sum + lp_16_koeffizienten(counter)*(to_integer(unsigned(data_ram(counter)))-2048);
	end if;
	
	if counter = filterordnung then
		data_out_new <= '1';
		data_out <= std_logic_vector(to_unsigned((sum*30)/2**17 + 2**11,12)); --unsigned output
		sum <= 0;
	end if;
	
end process;

end behav;